@@ -1,28 +1,26 @@
 #!/usr/bin/env python
-#
-# $Id$
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
 """OSX specific tests.  These are implicitly run by test_psutil.py."""
 
-import unittest
-import subprocess
-import time
-import sys
 import os
 import re
+import subprocess
+import sys
+import time
 
 import psutil
 
 from psutil._compat import PY3
-from test_psutil import reap_children, get_test_subprocess, sh
+from test_psutil import (MEMORY_TOLERANCE, OSX, sh, get_test_subprocess,
+                         reap_children, retry_before_failing, unittest,
+                         TRAVIS)
 
 
 PAGESIZE = os.sysconf("SC_PAGE_SIZE")
-TOLERANCE = 200 * 1024  # 200 KB
 
 
 def sysctl(cmdline):
@@ -38,6 +36,7 @@ def sysctl(cmdline):
     except ValueError:
         return result
 
+
 def vm_stat(field):
     """Wrapper around 'vm_stat' cmdline utility."""
     out = sh('vm_stat')
@@ -49,30 +48,52 @@ def vm_stat(field):
     return int(re.search('\d+', line).group(0)) * PAGESIZE
 
 
+# http://code.activestate.com/recipes/578019/
+def human2bytes(s):
+    SYMBOLS = {
+        'customary': ('B', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'),
+    }
+    init = s
+    num = ""
+    while s and s[0:1].isdigit() or s[0:1] == '.':
+        num += s[0]
+        s = s[1:]
+    num = float(num)
+    letter = s.strip()
+    for name, sset in SYMBOLS.items():
+        if letter in sset:
+            break
+    else:
+        if letter == 'k':
+            sset = SYMBOLS['customary']
+            letter = letter.upper()
+        else:
+            raise ValueError("can't interpret %r" % init)
+    prefix = {sset[0]: 1}
+    for i, s in enumerate(sset[1:]):
+        prefix[s] = 1 << (i+1)*10
+    return int(num * prefix[letter])
+
+
+@unittest.skipUnless(OSX, "not an OSX system")
 class OSXSpecificTestCase(unittest.TestCase):
 
-    def setUp(self):
-        self.pid = get_test_subprocess().pid
+    @classmethod
+    def setUpClass(cls):
+        cls.pid = get_test_subprocess().pid
 
-    def tearDown(self):
+    @classmethod
+    def tearDownClass(cls):
         reap_children()
 
-    def assert_eq_w_tol(self, first, second, tolerance):
-        difference = abs(first - second)
-        if difference <= tolerance:
-            return
-        msg = '%r != %r within %r delta (%r difference)' \
-              % (first, second, tolerance, difference)
-        raise AssertionError(msg)
-
     def test_process_create_time(self):
-        cmdline = "ps -o lstart -p %s" %self.pid
+        cmdline = "ps -o lstart -p %s" % self.pid
         p = subprocess.Popen(cmdline, shell=1, stdout=subprocess.PIPE)
         output = p.communicate()[0]
         if PY3:
             output = str(output, sys.stdout.encoding)
         start_ps = output.replace('STARTED', '').strip()
-        start_psutil = psutil.Process(self.pid).create_time
+        start_psutil = psutil.Process(self.pid).create_time()
         start_psutil = time.strftime("%a %b %e %H:%M:%S %Y",
                                      time.localtime(start_psutil))
         self.assertEqual(start_ps, start_psutil)
@@ -104,27 +125,44 @@ class OSXSpecificTestCase(unittest.TestCase):
             if abs(usage.used - used) > 10 * 1024 * 1024:
                 self.fail("psutil=%s, df=%s" % usage.used, used)
 
+    def test_cpu_count_logical(self):
+        num = sysctl("sysctl hw.logicalcpu")
+        self.assertEqual(num, psutil.cpu_count(logical=True))
+
+    def test_cpu_count_physical(self):
+        num = sysctl("sysctl hw.physicalcpu")
+        self.assertEqual(num, psutil.cpu_count(logical=False))
+
     # --- virtual mem
 
     def test_vmem_total(self):
         sysctl_hwphymem = sysctl('sysctl hw.memsize')
-        self.assertEqual(sysctl_hwphymem, psutil.TOTAL_PHYMEM)
+        self.assertEqual(sysctl_hwphymem, psutil.virtual_memory().total)
 
+    @unittest.skipIf(TRAVIS, "")
+    @retry_before_failing()
     def test_vmem_free(self):
         num = vm_stat("free")
-        self.assert_eq_w_tol(psutil.virtual_memory().free, num, TOLERANCE)
+        self.assertAlmostEqual(psutil.virtual_memory().free, num,
+                               delta=MEMORY_TOLERANCE)
 
+    @retry_before_failing()
     def test_vmem_active(self):
         num = vm_stat("active")
-        self.assert_eq_w_tol(psutil.virtual_memory().active, num, TOLERANCE)
+        self.assertAlmostEqual(psutil.virtual_memory().active, num,
+                               delta=MEMORY_TOLERANCE)
 
+    @retry_before_failing()
     def test_vmem_inactive(self):
         num = vm_stat("inactive")
-        self.assert_eq_w_tol(psutil.virtual_memory().inactive, num, TOLERANCE)
+        self.assertAlmostEqual(psutil.virtual_memory().inactive, num,
+                               delta=MEMORY_TOLERANCE)
 
+    @retry_before_failing()
     def test_vmem_wired(self):
         num = vm_stat("wired")
-        self.assert_eq_w_tol(psutil.virtual_memory().wired, num, TOLERANCE)
+        self.assertAlmostEqual(psutil.virtual_memory().wired, num,
+                               delta=MEMORY_TOLERANCE)
 
     # --- swap mem
 
@@ -137,18 +175,22 @@ class OSXSpecificTestCase(unittest.TestCase):
         self.assertEqual(psutil.swap_memory().sout, num)
 
     def test_swapmem_total(self):
-        tot1 = psutil.swap_memory().total
-        tot2 = 0
-        # OSX uses multiple cache files:
-        # http://en.wikipedia.org/wiki/Paging#OS_X
-        for name in os.listdir("/var/vm/"):
-            file = os.path.join("/var/vm", name)
-            if os.path.isfile(file):
-                tot2 += os.path.getsize(file)
-        self.assertEqual(tot1, tot2)
+        out = sh('sysctl vm.swapusage')
+        out = out.replace('vm.swapusage: ', '')
+        total, used, free = re.findall('\d+.\d+\w', out)
+        psutil_smem = psutil.swap_memory()
+        self.assertEqual(psutil_smem.total, human2bytes(total))
+        self.assertEqual(psutil_smem.used, human2bytes(used))
+        self.assertEqual(psutil_smem.free, human2bytes(free))
+
+
+def main():
+    test_suite = unittest.TestSuite()
+    test_suite.addTest(unittest.makeSuite(OSXSpecificTestCase))
+    result = unittest.TextTestRunner(verbosity=2).run(test_suite)
+    return result.wasSuccessful()
 
 
 if __name__ == '__main__':
-    test_suite = unittest.TestSuite()
-    test_suite.addTest(unittest.makeSuite(OSXSpecificTestCase))
-    unittest.TextTestRunner(verbosity=2).run(test_suite)
+    if not main():
+        sys.exit(1)
